import django.dispatch

post_save_message_request = django.dispatch.Signal(
    providing_args=['request', 'message']
)

post_save_message_response = django.dispatch.Signal(
    providing_args=['request', 'response', 'message']
)



from django.contrib import admin
from django.http import Http404, HttpResponse
from django.shortcuts import get_object_or_404
from roma import ReadOnlyModelAdmin

from wiretap.models import Message, Tap

class TapAdmin(admin.ModelAdmin):
    list_display = ('path_regex',)

class MessageAdmin(ReadOnlyModelAdmin):
    list_display = ('request', 'response', 'remote_addr', 'started_at', 'duration',)
    list_filter = ('started_at', 'req_method', 'res_status_code',)
    search_fields = ('remote_addr', 'req_path',)

    def request(self, obj):
        return unicode(obj)
    request.admin_order_field = 'req_path'

    def response(self, obj):
        if obj.res_status_code is not None:
            return '{} {}'.format(
                obj.res_status_code,
                obj.res_reason_phrase.encode('ascii', 'ignore')
            )
        else:
            return ''
    response.admin_order_field = 'res_status_code'

    def get_urls(self):
        from django.conf.urls import patterns, url

        return patterns('',
            url(
                r'^(?P<pk>\d+)/req-body/$',
                self.admin_site.admin_view(self.body_view),
                kwargs={
                    'field_name': 'req_body',
                    'get_header_name': 'get_req_header'
                }
            ),
            url(
                r'^(?P<pk>\d+)/res-body/$',
                self.admin_site.admin_view(self.body_view),
                kwargs={
                    'field_name': 'res_body',
                    'get_header_name': 'get_res_header'
                }
            )
        ) + super(MessageAdmin, self).get_urls()

    def body_view(self, request, pk, field_name, get_header_name):
        message = get_object_or_404(Message, id=pk)

        value = getattr(message, field_name)
        if not value:
            raise Http404()

        value.open()

        get_header = getattr(message, get_header_name)
        content_type = get_header('Content-Type', 'text/plain; charset=us-ascii')

        return HttpResponse(value.read(), content_type=content_type)

admin.site.register(Tap, TapAdmin)
admin.site.register(Message, MessageAdmin)



import json

from django.db import models

class _NotSet(object):
    pass

class Tap(models.Model):
    path_regex = models.CharField(max_length=255, blank=True)

    def __unicode__(self):
        return self.path_regex

class Message(models.Model):
    started_at = models.DateTimeField(db_index=True)
    ended_at = models.DateTimeField(blank=True, null=True)

    remote_addr = models.CharField(max_length=39, db_index=True)

    req_method = models.CharField(max_length=16)
    req_path = models.TextField()
    req_headers_json = models.TextField()
    req_body = models.FileField(blank=True, null=True, upload_to='message/%Y/%m/%d')

    res_status_code = models.PositiveIntegerField(blank=True, null=True)
    res_reason_phrase = models.CharField(max_length=64)
    res_headers_json = models.TextField()
    res_body = models.FileField(blank=True, null=True, upload_to='message/%Y/%m/%d')

    @property
    def duration(self):
        if self.ended_at is None:
            return None
        else:
            return (self.ended_at - self.started_at).total_seconds()

    @property
    def req_headers(self):
        return json.loads(self.req_headers_json)

    @property
    def res_headers(self):
        return json.loads(self.res_headers_json)

    def get_req_header(self, key, default=_NotSet):
        return self._get_header(self.req_headers, key, default)

    def get_res_header(self, key, default=_NotSet):
        return self._get_header(self.res_headers, key, default)

    def _get_header(self, headers, search_key, default):
        search_key = search_key.title()
        try:
            return next(
                value
                for (key, value) in headers
                if key == search_key
            )
        except StopIteration:
            if default is _NotSet:
                raise KeyError(search_key)
            else:
                return default

    def __unicode__(self):
        return u'{} {}'.format(
            self.req_method,
            self.req_path.encode('ascii', 'ignore')
        )



from django.core.exceptions import MiddlewareNotUsed
from django.test import RequestFactory, TestCase
from django.test.utils import override_settings

from wiretap.middleware import WiretapMiddleware
from wiretap.models import Message, Tap

class WiretapTestCase(TestCase):
    def setUp(self):
        self.request_factory = RequestFactory()

    @override_settings(DEBUG=True)
    def test_enabled_if_debugging(self):
        WiretapMiddleware()

    @override_settings(DEBUG=False)
    def test_disabled_if_not_debugging(self):
        with self.assertRaises(MiddlewareNotUsed):
            WiretapMiddleware()

    @override_settings(DEBUG=True)
    def test_no_taps(self):
        self.assertEqual(Tap.objects.count(), 0)
        WiretapMiddleware().process_request(self.request_factory.get('/'))
        self.assertEqual(Message.objects.count(), 0)

    @override_settings(DEBUG=True)
    def test_tap_match(self):
        Tap.objects.create(path_regex='/test')
        WiretapMiddleware().process_request(self.request_factory.get('/test'))
        self.assertEqual(Message.objects.count(), 1)

    @override_settings(DEBUG=True)
    def test_tap_mismatch(self):
        Tap.objects.create(path_regex='/test')
        WiretapMiddleware().process_request(self.request_factory.get('/real'))
        self.assertEqual(Message.objects.count(), 0)



#!/usr/bin/env python

import os.path
import sys

from setuptools import setup

sys.path.insert(0, os.path.join(os.path.dirname(__file__), 'testproject'))

setup(
    version='0.1.2',
    url='https://github.com/nathforge/django-wiretap',
    name='django-wiretap',
    description='https://github.com/nathforge/django-wiretap',
    long_description=open(os.path.join(os.path.dirname(__file__), 'README.rst')).read(),
    author='Nathan Reynolds',
    author_email='email@nreynolds.co.uk',
    packages=['wiretap'],
    package_dir={'': 'src'},
    install_requires=[
        'django >= 1.5, < 1.8',
        'django-roma',
        'six'
    ],
    test_suite='testproject.runtests.main',
    classifiers = [
        'Programming Language :: Python',
        'Programming Language :: Python :: 3'
    ]
)



import json
import re
import tempfile
import uuid

from django.conf import settings
from django.core.exceptions import MiddlewareNotUsed
from django.core.files.uploadedfile import SimpleUploadedFile, UploadedFile
from django.http import StreamingHttpResponse
from django.http.cookie import SimpleCookie
from django.utils import timezone
import six

from wiretap.models import Message, Tap
from wiretap.signals import post_save_message_request, post_save_message_response

DEFAULT_MAX_MESSAGE_COUNT = 1000

class WiretapMiddleware(object):
    """
    Wiretap middleware. Saves HTTP requests & responses to the `Message` model.
    """

    def __init__(self):
        """
        Initialise the middleware.
        """

        # Wiretap streams responses to disk, then stores the data in a
        # `FileField`.
        #
        # This isn't particularly performant, so we'll disable ourselves if
        # Django isn't in debug mode.
        if not getattr(settings, 'DEBUG', False):
            raise MiddlewareNotUsed()

    def should_tap(self, request):
        """
        Returns true if we should store the request/response.
        """

        matching_tap = False
        for tap in Tap.objects.all():
            if re.search(tap.path_regex, request.path):
                return True

        return False

    def process_request(self, request):
        """
        Process incoming requests. If we're tracking this request, we'll store
        a created `Message` object in `request.wiretap_message`.
        """

        request.wiretap_message = None

        if not self.should_tap(request):
            return

        req_started_at = timezone.now()

        req_content_type = None
        req_headers = []
        for (key, value) in six.iteritems(request.META):
            if key.startswith('HTTP_'):
                key = key[5:].title()
                req_headers.append((key, value))
                if key == 'Content-Type':
                    req_content_type = value

        if not request.body:
            req_body_file = None
        else:
            req_body_file = SimpleUploadedFile(
                name=str(uuid.uuid4()),
                content=request.body,
                content_type=req_content_type
            )

        request.wiretap_message = Message.objects.create(
            started_at=req_started_at,
            ended_at=timezone.now(),
            remote_addr=request.META['REMOTE_ADDR'],
            req_method=request.method,
            req_path=request.path,
            req_headers_json=json.dumps(req_headers),
            req_body=req_body_file
        )

        post_save_message_request.send(
            sender=self.__class__,
            request=request,
            message=request.wiretap_message
        )

    def process_response(self, request, response):
        """
        Process the response. If we're tracking this request,
        `request.wiretap_message` will be set.
        """

        if not request.wiretap_message:
            return response
        else:
            return WiretapHttpResponse(
                request,
                response,
                request.wiretap_message
            )

class WiretapHttpResponse(StreamingHttpResponse):
    """
    HttpResponse that wraps another response, mimicking it and saving the data
    to a `Message` object.
    """

    def __init__(self, request, response, message):
        super(WiretapHttpResponse, self).__init__()
        self._copy_from_response(response)
        self._request = request
        self._response = response
        self._message = message

    def _copy_from_response(self, response):
        """
        Copy data from the original response.
        """

        for key, _ in self.items():
            del self[key]

        for key, value in response.items():
            self[key] = value

        self.status_code = response.status_code
        self.reason_phrase = response.reason_phrase
        self.cookies = SimpleCookie(str(response.cookies))

    def __iter__(self):
        with tempfile.NamedTemporaryFile() as fp:
            for chunk in self._response:
                yield chunk
                fp.write(chunk)

            if fp.tell() == 0:
                res_body_file = None
            else:
                fp.seek(0)
                res_body_file = UploadedFile(
                    file=fp,
                    name=str(uuid.uuid4()),
                    content_type=self._response.get('content-type')
                )

            self._message.res_status_code = self._response.status_code
            self._message.res_reason_phrase = self._response.reason_phrase
            self._message.res_headers_json = json.dumps(self._response.items())
            self._message.res_body = res_body_file
            self._message.save()

        post_save_message_response.send(
            sender=self.__class__,
            request=self._request,
            response=self._response,
            message=self._message
        )

        delete_old_messages()

def delete_old_messages():
    """
    Deletes old `Message` objects.
    """

    max_message_count = getattr(
        settings,
        'WIRETAP_MAX_MESSAGE_COUNT',
        DEFAULT_MAX_MESSAGE_COUNT
    )

    if max_message_count is not None:
        delete_pks = Message.objects\
            .order_by('-started_at')\
            .values_list('pk', flat=True)\
            [max_message_count:]

        Message.objects\
            .filter(id__in=delete_pks)\
            .delete()
